
/* Chrysalide - Outil d'analyse de fichiers binaires
 * portions.c - navigation dans les portions de binaire
 *
 * Copyright (C) 2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "portions.h"


#include <i18n.h>


#include "tbitem-int.h"
#include "../../format/format.h"
#include "../../gtkext/gtkbinarystrip.h"



/* Elément réactif présentant des portions de binaire (instance) */
struct _GPortionsTbItem
{
    GToolbarItem parent;                    /* A laisser en premier        */

};


/* Elément réactif présentant des portions de binaire (classe) */
struct _GPortionsTbItemClass
{
    GToolbarItemClass parent;               /* A laisser en premier        */

};



/* Initialise la classe des éléments réactifs de l'éditeur. */
static void g_portions_tbitem_class_init(GPortionsTbItemClass *);

/* Initialise une instance d'élément réactif pour l'éditeur. */
static void g_portions_tbitem_init(GPortionsTbItem *);

/* Supprime toutes les références externes. */
static void g_portions_tbitem_dispose(GPortionsTbItem *);

/* Procède à la libération totale de la mémoire. */
static void g_portions_tbitem_finalize(GPortionsTbItem *);

/* Réagit à un changement du binaire courant. */
static void update_portions_item_binary(GEditorItem *, GLoadedBinary *);

/* Fait suivre un changement d'adresse dans la barre. */
static void track_address_on_binary_strip(GtkBinaryStrip *, GEditorItem *);



/* Indique le type défini pour un affichage de portions destiné à une barre d'outils. */
G_DEFINE_TYPE(GPortionsTbItem, g_portions_tbitem, G_TYPE_TOOLBAR_ITEM);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des éléments réactifs de l'éditeur.     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_portions_tbitem_class_init(GPortionsTbItemClass *klass)
{
    GObjectClass *object;                   /* Autre version de la classe  */
    GEditorItemClass *item;                 /* Encore une autre vision     */

    object = G_OBJECT_CLASS(klass);
    item = G_EDITOR_ITEM_CLASS(klass);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_portions_tbitem_dispose;
    object->finalize = (GObjectFinalizeFunc)g_portions_tbitem_finalize;

    item->update_binary = update_portions_item_binary;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : item = instance à initialiser.                               *
*                                                                             *
*  Description : Initialise une instance d'élément réactif pour l'éditeur.    *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_portions_tbitem_init(GPortionsTbItem *item)
{
    GtkWidget *widget;                      /* Composant principal         */
    GtkWidget *strip;                       /* Bande pour binaire          */

    widget = GTK_WIDGET(gtk_tool_item_new());
    gtk_tool_item_set_expand(GTK_TOOL_ITEM(widget), TRUE);
    gtk_widget_show(widget);

    G_EDITOR_ITEM(item)->widget = widget;

    strip = gtk_binary_strip_new();
    gtk_widget_show(strip);
    gtk_container_add(GTK_CONTAINER(widget), strip);

    g_object_set_data(G_OBJECT(widget), "strip", strip);

    g_signal_connect(strip, "select-address",
                     G_CALLBACK(track_address_on_binary_strip),
                     item);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : item = instance d'objet GLib à traiter.                      *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_portions_tbitem_dispose(GPortionsTbItem *item)
{
    G_OBJECT_CLASS(g_portions_tbitem_parent_class)->dispose(G_OBJECT(item));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : item = instance d'objet GLib à traiter.                      *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_portions_tbitem_finalize(GPortionsTbItem *item)
{
    G_OBJECT_CLASS(g_portions_tbitem_parent_class)->finalize(G_OBJECT(item));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : ref = espace de référencement global.                        *
*                                                                             *
*  Description : Crée un élément réactif présentant des portions de binaire.  *
*                                                                             *
*  Retour      : Adresse de la structure d'encadrement mise en place.         *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GEditorItem *create_portions_tb_item(GObject *ref)
{
    GPortionsTbItem *result;                /* Structure à retourner       */

    result = g_object_new(G_TYPE_PORTIONS_TBITEM, NULL);

    return g_toolbar_item_setup(G_TOOLBAR_ITEM(result), ref, "portions", _("Portions"));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : item   = élément réactif sollicité.                          *
*                binary = binaire chargé nouvellement affiché.                *
*                                                                             *
*  Description : Réagit à un changement du binaire courant.                   *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void update_portions_item_binary(GEditorItem *item, GLoadedBinary *binary)
{
    GtkBinaryStrip *strip;                  /* Bande pour binaire          */

    strip = GTK_BINARY_STRIP(g_object_get_data(G_OBJECT(item->widget), "strip"));

    gtk_binary_strip_attach(strip, binary);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : strip = composant d'affichage parcouru.                      *
*                addr  = nouvelle adresse du curseur courant.                 *
*                ref   = espace de référencement global.                      *
*                                                                             *
*  Description : Fait suivre un changement d'adresse dans la barre.           *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void track_address_on_binary_strip(GtkBinaryStrip *strip, GEditorItem *item)
{
    const vmpa2t *addr;                     /* Nouvelle destination        */
    GtkViewPanel *vpanel;                   /* Afficheur effectif de code  */

    addr = gtk_binary_strip_get_location(strip);

    vpanel = g_editor_item_get_current_view(item);
    gtk_view_panel_request_move(vpanel, addr);

    focus_address_in_editor_items(g_editor_item_get_current_binary(item), addr, item);

}
